﻿using System;
using System.IO;
using System.Linq;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;
using FluentValidation;
using InternshipSystem.Api.Options;
using InternshipSystem.Api.Queries;
using InternshipSystem.Api.Security;
using InternshipSystem.Core;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;
using Serilog;

namespace InternshipSystem.Api.Controllers
{
    [Route("access")]
    [ApiController]
    public class AccessController : ControllerBase
    {
        private readonly InternshipDbContext _context;
        private readonly JwtTokenService _tokenService;
        private readonly GutCasClient _loginClient;
        private readonly SecurityOptions _securityOptions;

        public AccessController(
            IOptions<SecurityOptions> options,
            InternshipDbContext context,
            JwtTokenService tokenService,
            GutCasClient loginClient)
        {
            _context = context;
            _tokenService = tokenService;
            _loginClient = loginClient;
            _securityOptions = options.Value;
        }
        

        [HttpGet("login")]
        public async Task<ActionResult> Authenticate(string code, CancellationToken cancellationToken)
        {
            var token =  await _loginClient.GetCasTokenAsync(code, cancellationToken);

            var casData = await _loginClient.GetProfileAsync(token, cancellationToken);

            if (!long.TryParse(casData.PersonNumber, out var id))
            {
                return BadRequest();
            }

            var student = await _context.Students.FirstOrDefaultAsync(s => s.Id == id, cancellationToken: cancellationToken);

            if (student == null)
            {
                student = CreateStudentWithCasData(casData);
                await _context.Students.AddAsync(student, cancellationToken);
                await _context.SaveChangesAsync(cancellationToken);
            }
            
            var identity = new ClaimsIdentity(new[]
            {
                new Claim(ClaimTypes.Name, student.FirstName),
                new Claim(ClaimTypes.Surname, student.LastName),
                new Claim(InternshipClaims.PersonNumber, student.Id.ToString()) 
            });

            return Ok(_tokenService.generateToken(identity));
        }

        [HttpGet("/dev/login")]
        public async Task<ActionResult> Authenticate(CancellationToken cancellationToken)
        {
            var identity = new ClaimsIdentity(new[]
            {
                new Claim(ClaimTypes.Name, "firstname"),
                new Claim(ClaimTypes.Surname, "lastname"),
                new Claim(InternshipClaims.PersonNumber, "1") 
            });
            
            return Ok(_tokenService.generateToken(identity));
        }

        [HttpGet("loginEdition")]
        [Authorize]
        public async Task<ActionResult> LoginIntoEdition(Guid editionId, [FromServices] User user, CancellationToken token)
        {
            var edition = await _context.Editions.FindAsync(editionId);

            var hasInternship = await _context.Entry(edition)
                .Collection(e => e.Internships)
                .Query()
                .AnyAsync(i => i.Student.Id == user.PersonNumber, token);

            if (!hasInternship)
            {
                return Unauthorized("Student isn't registered for this edition");
            }

            var newIdentity = User.Identities.First();
            
            newIdentity.AddClaim(new Claim(InternshipClaims.Edition, editionId.ToString()));

            return Ok(_tokenService.generateToken(newIdentity));
        }
        
        private Student CreateStudentWithCasData(CasUserData casData)
        {
            var id = long.Parse(casData.PersonNumber);
            var firstName = casData.FirstName;
            var lastName = casData.LastName;
            var email = casData.Mail.First(s => s.EndsWith("@student.pg.edu.pl"));
            var albumNumber = int.Parse(casData.AlbumNumber);

            return Student.CreateStudent(id, firstName, lastName, email, albumNumber);
        }
    }
}